#ifndef BMSBEL_RANDOM_MANAGER_H_
#define BMSBEL_RANDOM_MANAGER_H_

#include <vector>

#include "bms_bms.h"
#include "bms_handler.h"

class BmsRandomManager;


// -- BmsRandomElement ---------------------------------------------------
class BmsRandomElement {
public:
  virtual ~BmsRandomElement();
  virtual void Resolve( BmsRandomManager& manager, BmsBms& master ) const = 0;
  virtual std::string ToString( void ) const = 0;
};

// -- BmsRandomDataElement -------------------------------------
class BmsRandomDataElement : public BmsRandomElement {
public:
  explicit BmsRandomDataElement( void );

  virtual ~BmsRandomDataElement();

  BmsBms& GetData( void );

  virtual void Resolve( BmsRandomManager& manager, BmsBms& master ) const;

  virtual std::string ToString( void ) const;

private:
  BmsBms& data_;
};

// -- BmsRandomGenerateRandomElement -------------------------------------
class BmsRandomGenerateRandomElement : public BmsRandomElement {
public:
  explicit BmsRandomGenerateRandomElement( unsigned int max );

  unsigned int GetMax( void ) const;

  virtual void Resolve( BmsRandomManager& manager, BmsBms& master ) const;

  virtual std::string ToString( void ) const;

private:
  unsigned int max_;
};

// -- BmsRandomIfStatementElement ----------------------------------------
class BmsRandomIfStatementElement : public BmsRandomElement {
public:
  explicit BmsRandomIfStatementElement( unsigned int prerequisite );

  virtual ~BmsRandomIfStatementElement();

  unsigned int GetPrequisite( void ) const;

  BmsRandomDataElement&           MakeNewDataElement( void );
  BmsRandomGenerateRandomElement& MakeNewGenerateRandomElement( unsigned int max );
  BmsRandomIfStatementElement&    MakeNewIfStatementElement( unsigned int prerequisite );

  std::vector<BmsRandomElement*>& GetChildren( void );

  virtual void Resolve( BmsRandomManager& manager, BmsBms& master ) const;

  virtual std::string ToString( void ) const;

private:
  unsigned int                              prerequisite_;
  std::vector<BmsRandomElement*>            children_;
};


// -- BmsRandomManager ---------------------------------------------------
class BmsRandomManager {
public:
  explicit BmsRandomManager( BmsHandler::GenerateRandom& generate_random_handler );

  BmsRandomIfStatementElement& GetRoot( void );
  BmsRandomDataElement& GetRootData( void );

  unsigned int GetValue( void ) const;
  void SetValue( unsigned int value );

  unsigned int GenerateRandomValue( unsigned int max ) const;

  void Resolve( BmsBms& master );

  std::string ToString( void ) const;

private:
  BmsRandomIfStatementElement root_;
  BmsRandomDataElement&       root_data_;
  unsigned int                value_;

  BmsHandler::GenerateRandom& generate_random_handler_;
};

#endif // BMSBEL_RANDOM_MANAGER_H_
